from django.contrib import admin

from ..admin import GipsyMenu, ChildrenInline
from .models import GipsyDashboardMenu


class ChildrenDashboardInline(ChildrenInline):
    model = GipsyDashboardMenu
    exclude = ('icon',)


class GipsyDashboardMenuAdmin(GipsyMenu):
    inlines = [ChildrenDashboardInline]
    exclude = ('url', 'parent',)

admin.site.register(GipsyDashboardMenu, GipsyDashboardMenuAdmin)



from django import template
from django.contrib import admin
from django.core.urlresolvers import reverse
from django.db.models import Q

from gipsy.dashboard.models import GipsyDashboardMenu
from gipsy.dashboard.settings import GIPSY_DASHBOARD_URL,\
    GIPSY_VANILLA_INDEX_URL, GIPSY_THEME, GIPSY_DASHBOARD_TITLE,\
    GIPSY_DASHBOARD_CACHE_TIME   


register = template.Library()
tag_func = register.inclusion_tag('gipsy/dashboard/widgets/base.html',
                                  takes_context=True)


def get_active_url(request_formated):
    return GipsyDashboardMenu.objects\
        .filter(Q(url=request_formated) | Q(url=request_formated[:-1]))[:1]


@register.inclusion_tag('gipsy/dashboard/menu.html',
                        takes_context=True)
def gipsy_dashboard_menu(context, *args, **kwargs):
    """
    This tags manages the display of the admin menu.
    """
    items = []
    items_list = []
    items_parents = []
    parent_items_list = GipsyDashboardMenu.objects.filter(parent__isnull=True)\
        .order_by('order')
        
    # Only display menu items to user with the right permission
    for parent_item in parent_items_list:
        for child in parent_item.children:
            items_list.append(child)

    apps = {}

    try:
        # django >= 1.9
        app_list = admin.site.get_app_list(context['request'])
        for a in app_list:
            apps[a['app_url']] = a['models'][0]['perms']
    except AttributeError:
        # django < 1.9
        app_list = []
        for model, model_admin in admin.site._registry.items():
            app_label = model._meta.app_label
            module_perms = model_admin.get_model_perms(context['request'])
            app_url = reverse('admin:app_list', kwargs={'app_label': app_label}, current_app=admin.site.name)
            apps[app_url] = module_perms
    apps_keys = apps.keys()
    admin_prefix = reverse('admin:index').replace('/', '')
    for item in items_list:
        if item.url is None:
            continue
        url = item.url
        if url[0] != '/':
            url = '/' + item.url
        url_args = url.split('/')
        if admin_prefix == url_args[1]:
            permission = True
            for apps_key in apps_keys:
                if apps_key in url:
                    permission = False
                    for perm in apps[apps_key]:
                        if perm in url:
                            permission = apps[apps_key][perm]
                            break
                        if apps[apps_key][perm] == True:
                            permission = True
            if permission:
                items.append(item)
        else:
            items.append(item)

        if item in items and item.parent and not item.parent in items_parents:
            items_parents.append(item.parent)

    context['items_children'] = items
    context['items_parents'] = items_parents
    context['items'] = parent_items_list
    
    # Set the active item
    context['active'] = None
    if context['request'].path:
        active = []
        request_formated = context['request'].get_full_path()[1:]
        if request_formated:
            active = get_active_url(request_formated)

            # if nothing was found try to clean 'add' and 'edit' for generic urls
            # this is not done before to allow to point directly to those kinds of
            # urls
            if not active:
                if 'add' in request_formated:
                    request_formated = request_formated.split('/add', 1)
                if 'change' in request_formated:
                    request_formated = request_formated.split('/change', 1)[0].rsplit('/', 1)
                if 'history' in request_formated:
                    request_formated = request_formated.split('/history', 1)[0].rsplit('/', 1)

                active = get_active_url(request_formated[0] + '/')

    if len(active):
        context['active'] = active[0]
    context['dashboard_url'] = GIPSY_DASHBOARD_URL
    context['vanilla_index_url'] = GIPSY_VANILLA_INDEX_URL
    return context


@register.inclusion_tag('gipsy/dashboard/widgets/active_users.html')
def dashboard_active_users(count=0, title="CURRENTLY ACTIVE USERS",
                           label="CURRENT USERS"):
    return {'count': count, 'title': title, 'label': label}


@register.inclusion_tag('gipsy/dashboard/widgets/item_list.html')
def dashboard_item_list(items, title="MOST RECENT ITEMS"):
    return {'items': items, 'title': title}


@register.simple_tag
def gipsy_theme():
    """
    Returns the theme for the Admin-Interface.
    """
    return GIPSY_THEME


@register.simple_tag
def gipsy_title():
    """
    Returns the Title for the Admin-Interface.
    """
    return GIPSY_DASHBOARD_TITLE


@register.simple_tag(takes_context=True)
def url_active(context, viewname):
    request = context['request']
    current_path = request.path
    compare_path = reverse(viewname)
    if current_path == compare_path:
        return 'active'
    else:
        return ''


@register.assignment_tag(takes_context=True)
def gipsy_is_popup(context):
    """
    Checks if displayed in a popup. Addition to the django default
    templatetag as some shitty librairiesare not using this logic.
    """
    return (context.get('is_popup', False) or
            context['request'].GET.get('pop', False))


class GipsyDashboardCacheTime(template.Node):
    def __init__(self, varname):
        self.varname = varname

    def __repr__(self):
        return "<GipsyDashboardCachTime Node>"

    def render(self, context):
        context[self.varname] = GIPSY_DASHBOARD_CACHE_TIME
        return ''


@register.tag
def gipsy_dashboard_cache_time(parser, token):
    """
    Retrieves the cache time set in settings

    Usage::

        {% gipsy_dashboard_cache_time as [cache_time] %}
    """
    tokens = token.contents.split()
    if len(tokens) < 2:
        raise template.TemplateSyntaxError(
            "'gipsy_dashboard_cache_time' statements require at least one arguments")
    if tokens[1] != 'as':
        raise template.TemplateSyntaxError(
            "Second argument to 'gipsy_dashboard_cache_time' must be 'as'")

    return GipsyDashboardCacheTime(varname=tokens[2])


def gipsy_dashboard_widget(context, widget, index=None):
    """
    Template tag that renders a given dashboard module
    """
    context.update({
        'template': widget.template,
        'widget': widget,
        'index': index,
    })
    return context
gipsy_dashboard_widget = tag_func(gipsy_dashboard_widget)



from gipsy.dashboard.dashboard import Dashboard
from gipsy.dashboard.widgets import widgets


class DashboardDefault(Dashboard):
    """
    Defaut and static content class for dashboards.
    This class simply uses a render method where widgets are added. The widgets property is a list
    that can be appended as follow.
    """
    def render(self):
        # metrics evolution
        self.widgets.append(widgets.WidgetMetricsEvolution(
            title='daily visitors',
            label='visitors today',
            label_bottom='visitors difference since yesterday',
            result=2564,
            previous_result=2200,
        ))

        # metrics evolution
        self.widgets.append(widgets.WidgetMetricsEvolution(
            title='page views',
            label='page viewed today',
            label_bottom='pages viewed difference since yesterday',
            result=2564,
            previous_result=2200,
        ))

        # metrics single
        self.widgets.append(widgets.WidgetMetricsSingle(
            title='currently active users',
            label='active users',
            count=2564,
        ))

        # metrics single
        self.widgets.append(widgets.WidgetLineChart(
            title='our traffic overview',
            labels=['october', 'november', 'december'],
            values=[1027, 1060, 1500],
        ))

        # metrics list
        self.widgets.append(widgets.WidgetMetricsList(items=[
            {'icon': 'fa-file-image-o', 'label': 'posts', 'value': 75},
            {'icon': 'fa-comment-o', 'label': 'comments', 'value': 192},
            {'icon': 'fa-files-o', 'label': 'pages', 'value': 12},
            {'icon': 'fa-flag-o', 'label': 'in moderation', 'value': 4},
        ]))

        # model list
        self.widgets.append(widgets.WidgetModelList(items={}))

        # admin logs
        self.widgets.append(widgets.WidgetAdminLog())



# coding: utf-8

# DJANGO IMPORTS
from django.conf import settings

# Enable GIPSY admin site
GIPSY_ENABLE_ADMINSITE = getattr(settings, "GIPSY_ENABLE_ADMINSITE", True)


GIPSY_DASHBOARD_URL = getattr(settings, "GIPSY_DASHBOARD_URL", 'admin:index')
GIPSY_DASHBOARD_TITLE = getattr(settings, "GIPSY_DASHBOARD_TITLE", 'ADMINISTRATION')
GIPSY_DASHBOARD_HAS_ROWS = getattr(settings, "GIPSY_DASHBOARD_HAS_ROWS", True)
GIPSY_DASHBOARD_DEFAULT_GRID = getattr(settings, "GIPSY_DASHBOARD_DEFAULT_GRID", 12)
GIPSY_DASHBOARD_CACHE_TIME = int(getattr(settings, "GIPSY_DASHBOARD_CACHE_TIME", 500))
GIPSY_VANILLA_INDEX_URL = \
    getattr(settings, "GIPSY_VANILLA_INDEX_URL", 'admin:all_apps')
GIPSY_THEME = getattr(settings, "GIPSY_THEME",
                      getattr(settings, "STATIC_URL", "")
                      + 'gipsy_dashboard/css/gipsy.css')



from django.contrib import admin


class ChildrenInline(admin.TabularInline):
    sortable_field_name = "order"


class GipsyMenu(admin.ModelAdmin):
    inlines = [ChildrenInline]
    exclude = ('parent',)
    list_display = ['name', 'order']
    ordering = ['order']

    def get_queryset(self, request):
        """Overrides default queryset to only display parent items"""
        query = super(GipsyMenu, self).get_queryset(request)
        return query.filter(parent__isnull=True)



from functools import update_wrapper
from importlib import import_module

from django.apps import AppConfig
from django.contrib import admin
from django.contrib.admin.sites import AdminSite
from django.conf import settings
from django.conf.urls import url
from django.template.response import TemplateResponse


class DefaultAppConfig(AppConfig):
    name = 'gipsy.dashboard'

    def ready(self):
        class GipsyAdminSite(AdminSite):
            def get_urls(self):
                default_urlpatterns = super(GipsyAdminSite, self).get_urls()

                def wrap(view, cacheable=False):
                    def wrapper(*args, **kwargs):
                        return self.admin_view(view, cacheable)(*args,
                                                                **kwargs)
                    return update_wrapper(wrapper, view)

                urlpatterns = [
                    url(r'^$', wrap(self.dashboard), name='index'),
                    url(r'^all-apps$',
                        wrap(self.index),
                        name='all_apps')
                ]
                return urlpatterns + default_urlpatterns

            def init_dashboard_class(self):
                dashboard_module = getattr(
                    settings,
                    'GIPSY_DASHBOARD',
                    'gipsy.dashboard.presets.default.DashboardDefault'
                )
                mod, inst = dashboard_module.rsplit('.', 1)
                mod = import_module(mod)
                return getattr(mod, inst)

            def dashboard(self, request):
                """
                Displays the dashboard on the main page and triggers widget
                from the settings.GIPSY_DASHBOARD constant.
                """
                request.current_app = self.name
                context = dict(
                    dashboard=self.init_dashboard_class()(request),
                )
                return TemplateResponse(request, 'admin/dashboard.html',
                                        context)

        mysite = GipsyAdminSite()
        admin.site = mysite
        admin.sites.site = mysite
